#include "cpsp/HPThreadingOptions.hh"

#include <limits.h>

#include <biu/util/Util_String.h>

#include <ell/LatticeDescriptorCUB.hh>
#include <ell/LatticeDescriptorFCC.hh>

#include "cpsp/HCoreDatabaseFILE.hh"
#include "cpsp/HCoreDatabasePGSQL.hh"


namespace cpsp
{

	HPThreadingOptions::HPThreadingOptions(int argc, char** argv) 
		throw(cpsp::Exception) :
		sequence(NULL), latDescr(NULL), lattice(NULL), 
		coreSelection(FIRST), minE(0), maxE(0), maxSols(0),
		hullLvl(), maxPCoreDistance(0), coreDB(NULL),
		onlyCountingStructures(false), verboseOutput(false),
		minEvenOddHs(0)
		
	{
			// erlaubte Argumente initialisieren
		biu::OptionMap allowedArgs;
		std::string infoText;
		initAllowedArguments(allowedArgs,infoText);
		
			// programm argumente parsen	
		biu::COptionParser opts = biu::COptionParser(	allowedArgs, argc, 
														(char**)argv, infoText);
		
		if (opts.noErrors()) {
	
			if (opts.getBoolVal("help")) {
				throw cpsp::Exception(
					"",0);
			}
			
				// mode flags setzen
			verboseOutput = opts.getBoolVal("v");
			onlyCountingStructures = opts.getBoolVal("count");
			
				// sequence
			std::string tmpSeq = opts.getStrVal("seq");
			sequence = new std::string(biu::util::Util_String::str2upperCase(tmpSeq));
				// gueltigkeit pruefen
			ell::Alphabet hpAlph("HP",1);
			if (!hpAlph.isAlphabetString(*sequence) || sequence->size() < 3) {
				throw cpsp::Exception(
					"Error in arguments: sequence is no valid HP-sequence",-2);
			}
						
				// lattice modell
			if (opts.getStrVal("lat") == "CUB")
				latDescr = new ell::LatticeDescriptorCUB();
			else if (opts.getStrVal("lat") == "FCC")
				latDescr = new ell::LatticeDescriptorFCC();
				
			lattice = new ell::LatticeFrame(latDescr, 2*sequence->size()+4);
			
				// core selection mode
				// <all> ueberschreibt <allbest> wenn beide vorhanden
			if (opts.getBoolVal("all")) {
				coreSelection = ALL;
			} else if (opts.getBoolVal("allbest")) {
				coreSelection = ALL_BEST;
			}
			if (coreSelection == FIRST) {
				maxSols = 1;
			}
				// energie schranken
			minE = opts.getIntVal("minE");
			maxE = opts.getIntVal("maxE");
				// semantische fehler pruefen
			if (minE > maxE)
				throw cpsp::Exception("Error in arguments : minE > maxE", -2);
				
				// maximale strukturzahl
			if (coreSelection != FIRST) {
				int tmp = opts.getIntVal("maxSol");
				if (tmp < 1)
					throw cpsp::Exception("Error in arguments : maxSol <= 0", -2);
				maxSols = (unsigned int) tmp;
			}
			
			unsigned int i=0, pHead=0, pTail=0, pSubSeq=0, actLvl=0;
				// p-kopf und p-schwanz bestimmen
			pHead = sequence->find_first_not_of('P');
			pTail = sequence->find_last_of('P');
			pTail = (pTail == std::string::npos)? 0 : sequence->size() - pTail;
			
				// core size init
			unsigned int coreSize=(pHead<sequence->size())?1:0;
			if (coreSize == 1 && pHead%2==0)	// dann erste H-pos gerade
				minEvenOddHs = 1;
				// huellen level
			hullLvl = std::vector<unsigned int>(sequence->size(),0);	// init
				// p-kopf setzen
			for (i=0; i<pHead; i++) {
				hullLvl[i] = pHead-i;
			}
				// erster hullLvl init
			for (i=pHead+1; i<sequence->size(); i++) {
				if (sequence->at(i) == 'H') {				// ende von p-subseq
					pSubSeq = std::max(pSubSeq, actLvl);	// p-subseq setzen
					actLvl = 0;								// reinit
					coreSize++;								// kerngroesse ++
					if (i%2==0) minEvenOddHs++;				// gerade H position
				} else {
					actLvl++;
				}
				hullLvl[i] = actLvl;			// level setzen
			}
				// rueckwaerts angleichen
			for (i=sequence->size()-pTail-1; i>pHead+1; i--) {
				if (hullLvl[i-1] > (hullLvl[i]+1)) {
					hullLvl[i-1] = hullLvl[i]+1;
				}
			}
				// min von even und odd H positionen bestimmen
			minEvenOddHs = std::min( minEvenOddHs, coreSize-minEvenOddHs);
			
				// singlet positionen bestimmen
				// P-singlets
			i = sequence->find("PHP");
			seqFeatureMap[H_SINGLET].clear();
			while ( i != std::string::npos ) {
				seqFeatureMap[H_SINGLET].push_back(i+1);
				i = sequence->find("PHP",i+2);
			}
				// h-singlets
			i = sequence->find("HPH");
			seqFeatureMap[P_SINGLET].clear();
			while ( i !=  std::string::npos ) {
				seqFeatureMap[P_SINGLET].push_back(i+1);
				i = sequence->find("HPH",i+2);
			}
			
				// sequence internal HH-contacts
			contactsInSeq = 0;
			i = sequence->find("HH");
			while ( i !=  std::string::npos ) {
				contactsInSeq++;
				i = sequence->find("HH", i+1);
			}
			
			
			
				// lattice frame size setzen
			maxPCoreDistance = 2* (	std::max(	(pSubSeq/2) +1, 
												std::max( pTail, pHead))
									+2);

				// H-core database
//			coreDB = new HCoreDatabasePGSQL("isis.informatik.uni-freiburg.de");

			if (opts.argExist("dbPath")) {
//				coreDB = new HCoreDatabaseFILE("e:/Research/CPP/IlogThreading/CoreData/");
				coreDB = new HCoreDatabaseFILE(opts.getStrVal("dbPath"));
			} else {
				throw cpsp::Exception(
					"H-core database Error: no database available (FILE or SQL)",-1);
			}
				// testen
			if (!coreDB->isConnected()) {
				throw cpsp::Exception(
					"H-core database Error: can't open database",-1);
			}
			if (!coreDB->initCoreAccess(	*latDescr,
											coreSize,
											-maxE,
											-minE)) {
				throw cpsp::Exception(
					"H-core database Error: access init failed",-1);
			}
		} else {
			throw cpsp::Exception(
				"Parameter Error: not parseable",-1);
		}
	}
	
	HPThreadingOptions::~HPThreadingOptions() 
	{
			// aufraeumen
		if( coreDB != NULL) {
			delete coreDB; coreDB = NULL;
		}
		if( lattice != NULL ) {
			delete lattice; lattice = NULL;
		}
		if( latDescr != NULL ) {
			delete latDescr; latDescr = NULL;
		}
		if( sequence != NULL ) {
			delete sequence; sequence = NULL;
		}
	}
	
	void
	HPThreadingOptions::initAllowedArguments(biu::OptionMap & allowedArgs,
											std::string &infoText ) const {
		allowedArgs.push_back(biu::COption(	
								"dbPath", false, biu::COption::STRING, 
								"file based database root path"));
		allowedArgs.push_back(biu::COption(	
								"seq", false, biu::COption::STRING, 
								"HP-sequence to fold"));
		allowedArgs.push_back(biu::COption(	
								"lat", true, biu::COption::STRING, 
								"lattice (CUB, FCC)", "CUB"));
		allowedArgs.push_back(biu::COption(	
								"all", true, biu::COption::BOOL, 
								"search all solutions"));
		allowedArgs.push_back(biu::COption(	
								"allbest", true, biu::COption::BOOL, 
								"search all solutions with best energy"));
		allowedArgs.push_back(biu::COption(	
								"minE", true, biu::COption::INT, 
					"lower energy bound for the search (value <= maxE <= 0)",
								biu::util::Util_String::int2str(INT_MIN+2)));
		allowedArgs.push_back(biu::COption(	
								"maxE", true, biu::COption::INT, 
							"upper energy bound for the search (value <= 0)",
								"0"));
		allowedArgs.push_back(biu::COption(	
								"maxSol", true, biu::COption::INT, 
							"maximum of solutions to search for (value > 0)",
								biu::util::Util_String::int2str(INT_MAX-2)));
		allowedArgs.push_back(biu::COption(	
								"count", true, biu::COption::BOOL, 
								"only counting structures"));
		allowedArgs.push_back(biu::COption(	
								"v", true, biu::COption::BOOL, 
								"verbose output"));
		allowedArgs.push_back(biu::COption(	
								"help", true, biu::COption::BOOL, 
								"program parameters and help"));

		infoText = " - the option <all> overwrites the <allbest> option";
	
	} // initArguments

} // namespace cpsp
